@@ -13,9 +13,9 @@ use crate::{
 
 use crossbeam_channel::{Sender, TrySendError};
 use std::{
+    any::TypeId,
     borrow::Borrow,
     collections::hash_map::RandomState,
-    error::Error,
     future::Future,
     hash::{BuildHasher, Hash},
     sync::Arc,
@@ -57,7 +57,7 @@ use std::{
 /// // Cargo.toml
 /// //
 /// // [dependencies]
-/// // moka = { version = "0.5", features = ["future"] }
+/// // moka = { version = "0.6", features = ["future"] }
 /// // tokio = { version = "1", features = ["rt-multi-thread", "macros" ] }
 /// // futures = "0.3"
 ///
@@ -285,7 +285,7 @@ where
     /// // Cargo.toml
     /// //
     /// // [dependencies]
-    /// // moka = { version = "0.5", features = ["future"] }
+    /// // moka = { version = "0.6", features = ["future"] }
     /// // futures = "0.3"
     /// // tokio = { version = "1", features = ["rt-multi-thread", "macros" ] }
     /// use moka::future::Cache;
@@ -329,7 +329,7 @@ where
     ///
     /// **A Sample Result**
     ///
-    /// - The async black resolved exactly once by task 3.
+    /// - The `init` future (async black) was resolved exactly once by task 3.
     /// - Other tasks were blocked until task 3 inserted the value.
     ///
     /// ```console
@@ -344,6 +344,14 @@ where
     /// Task 2 got the value. (len: 10485760)
     /// ```
     ///
+    /// # Panics
+    ///
+    /// This method panics when the `init` future has been panicked. When it happens,
+    /// only the caller whose `init` future panicked will get the panic (e.g. only
+    /// task 3 in the above sample). If there are other calls in progress (e.g. task
+    /// 0, 1 and 2 above), this method will restart and resolve one of the remaining
+    /// `init` futures.
+    ///
     pub async fn get_or_insert_with<F>(&self, key: K, init: F) -> V
     where
         F: Future<Output = V> + Send + 'static,
@@ -359,8 +367,8 @@ where
     ///
     /// This method prevents to resolve the init future multiple times on the same
     /// key even if the method is concurrently called by many async tasks; only one
-    /// of the calls resolves its future, and other calls wait for that future to
-    /// complete.
+    /// of the calls resolves its future (as long as these futures return the same
+    /// error type), and other calls wait for that future to complete.
     ///
     /// # Example
     ///
@@ -368,17 +376,14 @@ where
     /// // Cargo.toml
     /// //
     /// // [dependencies]
-    /// // moka = { version = "0.5", features = ["future"] }
+    /// // moka = { version = "0.6", features = ["future"] }
     /// // futures = "0.3"
     /// // reqwest = "0.11"
     /// // tokio = { version = "1", features = ["rt-multi-thread", "macros" ] }
     /// use moka::future::Cache;
     ///
     /// // This async function tries to get HTML from the given URI.
-    /// async fn get_html(
-    ///     task_id: u8,
-    ///     uri: &str,
-    /// ) -> Result<String, Box<dyn std::error::Error + Send + Sync + 'static>> {
+    /// async fn get_html(task_id: u8, uri: &str) -> Result<String, reqwest::Error> {
     ///     println!("get_html() called by task {}.", task_id);
     ///     Ok(reqwest::get(uri).await?.text().await?)
     /// }
@@ -423,7 +428,7 @@ where
     ///
     /// **A Sample Result**
     ///
-    /// - `get_html()` called exactly once by task 2.
+    /// - `get_html()` was called exactly once by task 2.
     /// - Other tasks were blocked until task 2 inserted the value.
     ///
     /// ```console
@@ -438,17 +443,18 @@ where
     /// Task 3 got the value. (len: 19419)
     /// ```
     ///
-    #[allow(clippy::redundant_allocation)]
-    // https://rust-lang.github.io/rust-clippy/master/index.html#redundant_allocation
-    // `Arc<Box<dyn ..>>` in the return type creates an extra heap allocation.
-    // This will be addressed by Moka v0.6.0.
-    pub async fn get_or_try_insert_with<F>(
-        &self,
-        key: K,
-        init: F,
-    ) -> Result<V, Arc<Box<dyn Error + Send + Sync + 'static>>>
+    /// # Panics
+    ///
+    /// This method panics when the `init` future has been panicked. When it happens,
+    /// only the caller whose `init` future panicked will get the panic (e.g. only
+    /// task 2 in the above sample). If there are other calls in progress (e.g. task
+    /// 0, 1 and 3 above), this method will restart and resolve one of the remaining
+    /// `init` futures.
+    ///
+    pub async fn get_or_try_insert_with<F, E>(&self, key: K, init: F) -> Result<V, Arc<E>>
     where
-        F: Future<Output = Result<V, Box<dyn Error + Send + Sync + 'static>>> + Send + 'static,
+        F: Future<Output = Result<V, E>> + Send + 'static,
+        E: Send + Sync + 'static,
     {
         let hash = self.base.hash(&key);
         let key = Arc::new(key);
@@ -624,7 +630,8 @@ where
             InitResult::Initialized(v) => {
                 self.insert_with_hash(Arc::clone(&key), hash, v.clone())
                     .await;
-                self.value_initializer.remove_waiter(&key);
+                self.value_initializer
+                    .remove_waiter(&key, TypeId::of::<()>());
                 v
             }
             InitResult::ReadExisting(v) => v,
@@ -632,18 +639,15 @@ where
         }
     }
 
-    #[allow(clippy::redundant_allocation)]
-    // https://rust-lang.github.io/rust-clippy/master/index.html#redundant_allocation
-    // `Arc<Box<dyn ..>>` in the return type creates an extra heap allocation.
-    // This will be addressed by Moka v0.6.0.
-    async fn get_or_try_insert_with_hash_and_fun<F>(
+    async fn get_or_try_insert_with_hash_and_fun<F, E>(
         &self,
         key: Arc<K>,
         hash: u64,
         init: F,
-    ) -> Result<V, Arc<Box<dyn Error + Send + Sync + 'static>>>
+    ) -> Result<V, Arc<E>>
     where
-        F: Future<Output = Result<V, Box<dyn Error + Send + Sync + 'static>>>,
+        F: Future<Output = Result<V, E>>,
+        E: Send + Sync + 'static,
     {
         if let Some(v) = self.base.get_with_hash(&key, hash) {
             return Ok(v);
@@ -658,7 +662,8 @@ where
                 let hash = self.base.hash(&key);
                 self.insert_with_hash(Arc::clone(&key), hash, v.clone())
                     .await;
-                self.value_initializer.remove_waiter(&key);
+                self.value_initializer
+                    .remove_waiter(&key, TypeId::of::<E>());
                 Ok(v)
             }
             InitResult::ReadExisting(v) => Ok(v),
@@ -761,7 +766,7 @@ mod tests {
     use crate::{common::time::Clock, future::CacheBuilder};
 
     use async_io::Timer;
-    use std::time::Duration;
+    use std::{convert::Infallible, sync::Arc, time::Duration};
 
     #[tokio::test]
     async fn basic_single_async_task() {
@@ -1173,6 +1178,15 @@ mod tests {
 
     #[tokio::test]
     async fn get_or_try_insert_with() {
+        use std::sync::Arc;
+
+        // Note that MyError does not implement std::error::Error trait
+        // like anyhow::Error.
+        #[derive(Debug)]
+        pub struct MyError(String);
+
+        type MyResult<T> = Result<T, Arc<MyError>>;
+
         let cache = Cache::new(100);
         const KEY: u32 = 0;
 
@@ -1189,7 +1203,7 @@ mod tests {
                     .get_or_try_insert_with(KEY, async {
                         // Wait for 300 ms and return an error.
                         Timer::after(Duration::from_millis(300)).await;
-                        Err("task1 error".into())
+                        Err(MyError("task1 error".into()))
                     })
                     .await;
                 assert!(v.is_err());
@@ -1205,7 +1219,7 @@ mod tests {
             async move {
                 // Wait for 100 ms before calling `get_or_try_insert_with`.
                 Timer::after(Duration::from_millis(100)).await;
-                let v = cache2
+                let v: MyResult<_> = cache2
                     .get_or_try_insert_with(KEY, async { unreachable!() })
                     .await;
                 assert!(v.is_err());
@@ -1222,7 +1236,7 @@ mod tests {
             async move {
                 // Wait for 400 ms before calling `get_or_try_insert_with`.
                 Timer::after(Duration::from_millis(400)).await;
-                let v = cache3
+                let v: MyResult<_> = cache3
                     .get_or_try_insert_with(KEY, async {
                         // Wait for 300 ms and return an Ok(&str) value.
                         Timer::after(Duration::from_millis(300)).await;
@@ -1241,7 +1255,7 @@ mod tests {
             async move {
                 // Wait for 500 ms before calling `get_or_try_insert_with`.
                 Timer::after(Duration::from_millis(500)).await;
-                let v = cache4
+                let v: MyResult<_> = cache4
                     .get_or_try_insert_with(KEY, async { unreachable!() })
                     .await;
                 assert_eq!(v.unwrap(), "task3");
@@ -1258,7 +1272,7 @@ mod tests {
             async move {
                 // Wait for 800 ms before calling `get_or_try_insert_with`.
                 Timer::after(Duration::from_millis(800)).await;
-                let v = cache5
+                let v: MyResult<_> = cache5
                     .get_or_try_insert_with(KEY, async { unreachable!() })
                     .await;
                 assert_eq!(v.unwrap(), "task3");
@@ -1303,4 +1317,55 @@ mod tests {
 
         futures::join!(task1, task2, task3, task4, task5, task6, task7, task8);
     }
+
+    #[tokio::test]
+    // https://github.com/moka-rs/moka/issues/43
+    async fn handle_panic_in_get_or_insert_with() {
+        use tokio::time::{sleep, Duration};
+
+        let cache = Cache::new(16);
+        let semaphore = Arc::new(tokio::sync::Semaphore::new(0));
+        {
+            let cache_ref = cache.clone();
+            let semaphore_ref = semaphore.clone();
+            tokio::task::spawn(async move {
+                let _ = cache_ref
+                    .get_or_insert_with(1, async move {
+                        semaphore_ref.add_permits(1);
+                        sleep(Duration::from_millis(50)).await;
+                        panic!("Panic during get_or_try_insert_with");
+                    })
+                    .await;
+            });
+        }
+        let _ = semaphore.acquire().await.expect("semaphore acquire failed");
+        assert_eq!(cache.get_or_insert_with(1, async { 5 }).await, 5);
+    }
+
+    #[tokio::test]
+    // https://github.com/moka-rs/moka/issues/43
+    async fn handle_panic_in_get_or_try_insert_with() {
+        use tokio::time::{sleep, Duration};
+
+        let cache = Cache::new(16);
+        let semaphore = Arc::new(tokio::sync::Semaphore::new(0));
+        {
+            let cache_ref = cache.clone();
+            let semaphore_ref = semaphore.clone();
+            tokio::task::spawn(async move {
+                let _ = cache_ref
+                    .get_or_try_insert_with(1, async move {
+                        semaphore_ref.add_permits(1);
+                        sleep(Duration::from_millis(50)).await;
+                        panic!("Panic during get_or_try_insert_with");
+                    })
+                    .await as Result<_, Arc<Infallible>>;
+            });
+        }
+        let _ = semaphore.acquire().await.expect("semaphore acquire failed");
+        assert_eq!(
+            cache.get_or_try_insert_with(1, async { Ok(5) }).await as Result<_, Arc<Infallible>>,
+            Ok(5)
+        );
+    }
 }
